
/******************************************************************************
*
* project name:    VTI-LOGGER
* project version: 1.1
* file name:       VTILOG.CPP
* file version:    1.1
* initial date:    15/03/2000
* author:          Thomas Nussbaumer (thomas.nussbaumer@gmx.net)
*
* modified:        13/02/2002 by Hai Pho Truong (novaxius@yahoo.ca)
* 
* description:     implementation of vti logger
*
* NOTICE: The credits goes completely to Rusty Wagner, because he released
*         the source codes of VTI. Otherwise tools like this wouldn't be
*         be possible !!!
*
*         Rusty Wagner can be contacted at rusty@acz.org
*
*         This project is currently maintained by Hai Pho Truong with the 
*         permission of Thomas Nussbaumer.
*
*
* The principle of the virtual link of VTI is using memory mapped
* files for the communication and some special window message for
* things like "saying hello and goodbye" and enable/disable the link.
*
* To fool VTI to think we're another VTI instance is done by
* registering our main window with the same window class which VTI uses
* ("TEmuWnd")
*
* ATTENTION: VTI-LOGGER CANNOT BE USED TO RECEIVE ANY DATA WHICH
*            NEEDS SOME HANDSHAKE HANDLING.
*
*            THEREFORE IF YOU'LL TRY TO SENT A VARIABLE VIA THE VIRTUAL
*            LINK PORT NOTHING HAPPENS, BECAUSE THE USED PROTOCOL FOR
*            VARIABLE SENDING DEPENDS ON HANDSHAKE HANDLING.
*
*            VTI-LOGGER SHOULD ONLY BE SEEN AS DEBUGGING HELP FOR PROGRAMMERS,
*            WHO SENT SAME DATA DIRECTLY TO THE LINK PORT WITHOUT AWAITING
*            ANY ANSWERS FROM THE LINK PORT
*

/****** UPDATE LOG ****************************************************************

13/02/2002 - Version 1.1

    + On-screen output or file ouput (selectable)
    + New GUI
    + Logfile can be openned automatically after the logging process is stopped
    + Implemented the workaround for the virtual link's bug: when VTI is started after 
      VTI-Logger, the virtual link would be broken. The solution is to enable the link 
      right after the reception of WM_HELLO from VTI. The same bug could be observed 
      when using 2 instances of VTI, only the second instance could activate the 
      virtual link.


15/03/2000 - Version 1.0

    Original version by Thomas Nussbaumer

/******************************************************************************/

#include "stdafx.h"
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <commdlg.h>
#include "resource.h"

//-----------------------------------------------------------------------------
// messages sent by VTI 2.5 beta 5
//-----------------------------------------------------------------------------

#define WM_HELLO        WM_USER+101
#define WM_GOODBYE      WM_USER+102
#define WM_ENABLE_LINK  WM_USER+110
#define WM_DISABLE_LINK WM_USER+111
#define WM_SEND_BUFFER  WM_USER+120

//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------

#define WM_USER_MINIMIZE    WM_USER+301
#define WM_USER_MAXIMIZE    WM_USER+302
#define WM_USER_RESTORE     WM_USER+303

#define SC_ABOUT           WM_USER+401


#define MAX_TEXT_BUFFER 0xFFFF      // 64K for WinNT & Win9x
#define MIN_TEXT_BUFFER 0x2000


//-----------------------
// Virtual link structure
//
typedef struct
{
    unsigned char buf[256];
    int start;
    int end;
} LinkBuffer;


//-----------------------
// Global
//
HINSTANCE hInst;
HWND hwndMain;

HWND      hdlg           = NULL;
HWND      otherWnd       = NULL;
HANDLE    hVLinkFileMap  = NULL;
HANDLE    hMutex         = NULL;
char      vLinkFileName[32];
char      logfile[1024]  = "";
FILE*     fp             = NULL;
UINT      mytimer        = 0;
BOOL	  already_reading = FALSE;
BOOL      vti_started_first;
BOOL      logging_enabled = FALSE;

WORD      lastWidth = 0;
WORD      lastHeight = 0;
WORD      minWidth = 0;
WORD      minHeight = 0;

LinkBuffer *vSendBuf,*vRecvBuf;

OPENFILENAME openDlgData;

//---------------

LRESULT CALLBACK MainWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);
BOOL CALLBACK DlgProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AboutProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

//-------------------------------------------
//
//
static BOOL InitApplication(void) {
	WNDCLASS wc;
	memset(&wc,0,sizeof(WNDCLASS));	
	wc.style = CS_HREDRAW|CS_VREDRAW |CS_DBLCLKS ;
	wc.lpfnWndProc = (WNDPROC)MainWndProc;
	wc.hInstance = hInst;
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszClassName = "TEmuWnd";   // trick VTI that it thinks we are another one !!
	wc.lpszMenuName = NULL;
	wc.hCursor = LoadCursor(NULL,IDC_ARROW);
	wc.hIcon = LoadIcon(hInst,(LPCTSTR)IDI_VTILOG);
	return RegisterClass(&wc);
}


//-------------------------------------------
//
//
void SetDlgItemPos(HWND hItem, SHORT deltaX, SHORT deltaY, SHORT deltaW, SHORT deltaH){
    RECT rect;
    POINT p1,p2;
    GetWindowRect(hItem,&rect);
    p1.x = rect.left;
    p1.y = rect.top;
    p2.x = rect.right-rect.left;
    p2.y = rect.bottom-rect.top;
    ScreenToClient(hdlg,&p1);       
    SetWindowPos(hItem,0,p1.x+deltaX,p1.y+deltaY,p2.x+deltaW,p2.y+deltaH,
                 SWP_NOREPOSITION|SWP_NOCOPYBITS);
}


//-------------------------------------------
//
//
void ActivateLink()
{
    if (IsDlgButtonChecked(hdlg,IDC_CHK_VTILINK_ENABLED)!=BST_CHECKED)
    {
        CheckDlgButton(hdlg,IDC_CHK_VTILINK_ENABLED,TRUE);
        mytimer = SetTimer(hwndMain,1,100,NULL);                
    }
}


//-------------------------------------------
//
//
void DeactivateLink()
{    
    if (IsDlgButtonChecked(hdlg,IDC_CHK_VTILINK_ENABLED)==BST_CHECKED)
    {
        CheckDlgButton(hdlg,IDC_CHK_VTILINK_ENABLED,FALSE);
        if (mytimer != 0) 
        {
            KillTimer(hwndMain,mytimer);                
            mytimer=0;
        }
    }
}


//-------------------------------------------
//
//
HWND CreateMainWindow(void) {
	return CreateWindow(
		"TEmuWnd",              // will trick VTI
	    "VTI-Logger",
		WS_POPUPWINDOW | WS_MINIMIZEBOX | WS_MAXIMIZEBOX,
		0,0,0,0,
		NULL,
		NULL,
		hInst,
		NULL);
}


//-------------------------------------------
//
//
LRESULT CALLBACK MainWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
	switch (msg) 
    {
        case WM_GETMINMAXINFO:
            {
                MINMAXINFO* pMinMax = (MINMAXINFO*) lParam;
                pMinMax->ptMaxSize.x = 0;
                pMinMax->ptMaxSize.y = 0;
                pMinMax->ptMaxTrackSize.x = 0;
                pMinMax->ptMaxTrackSize.y = 0;
                pMinMax->ptMinTrackSize.x = 0;
                pMinMax->ptMinTrackSize.y = 0;
            }
            break;

	    case WM_DESTROY:
            if (mytimer != 0) KillTimer(hwndMain,mytimer);
	        mytimer = 0;

            if (otherWnd != NULL) 
            {
                SendMessage(otherWnd,WM_DISABLE_LINK,0,0);
                SendMessage(otherWnd,WM_GOODBYE,0,0);
                otherWnd = NULL;
            }

		    if (fp != NULL) fclose(fp);
		    fp = NULL;
		    PostQuitMessage(0);
		    break;

        case WM_USER_MAXIMIZE:            
            return DefWindowProc(hwnd,WM_SYSCOMMAND,SC_MAXIMIZE,lParam);

        case WM_USER_MINIMIZE:
            return DefWindowProc(hwnd,WM_SYSCOMMAND,SC_MINIMIZE,lParam);

        case WM_USER_RESTORE:
            return DefWindowProc(hwnd,WM_SYSCOMMAND,SC_RESTORE,lParam);

        case WM_SYSCOMMAND:
            switch (wParam)
            {
                case SC_ABOUT:
                case SC_SIZE:
                case SC_MOVE:
                    SendMessage(hdlg,msg,wParam,lParam);
                    break;

                case SC_RESTORE:
                case SC_MAXIMIZE:
                case SC_MINIMIZE:                
                    SendMessage(hdlg,msg,wParam,lParam);
                    break;

                default:
                    return DefWindowProc(hwnd,msg,wParam,lParam);
            }
            break;

	    case WM_TIMER:
	        if (logging_enabled) 
            {
		        if (!already_reading) 
                {
			        already_reading=TRUE;

                    if ((IsDlgButtonChecked(hdlg,IDC_CHK_VTILINK_ENABLED)==BST_CHECKED) && 
                        (vRecvBuf != NULL)) 
                    {                        
                        int i=0;
                        char linkBuffer[513];
                        while ((vRecvBuf->start!=vRecvBuf->end) && (i<512))
                        {
                            // copy data to link buffer
                            linkBuffer[i] = vRecvBuf->buf[vRecvBuf->start];
                            vRecvBuf->start=(vRecvBuf->start+1)&255;
                            i++;
				        }                        
                        
                        if (i>0) 
                        {
                            linkBuffer[i] = 0;

                            // output to file
                            if (fp != NULL)     
                            {
                                fputs(linkBuffer,fp);
                                fflush(fp);
                            }

                            // ouput to screen
                            if (IsDlgButtonChecked(hdlg,IDC_CHK_SCREEN)==BST_CHECKED)
                            {
                                char* localBuffer = (char*) malloc(MAX_TEXT_BUFFER+1);
                                int oldSize = GetDlgItemText(hdlg,IDC_TXT_LOG,localBuffer,MAX_TEXT_BUFFER);
                                int newSize = strlen(linkBuffer);

                                if (newSize + oldSize >= MAX_TEXT_BUFFER)
                                    strcpy(localBuffer,localBuffer+newSize+MIN_TEXT_BUFFER);
                                strcat(localBuffer,linkBuffer);
                                SetDlgItemText(hdlg,IDC_TXT_LOG,localBuffer);
                                free(localBuffer);

                                // scroll the log window to the end                                                                
                                SendMessage(GetDlgItem(hdlg,IDC_TXT_LOG),EM_LINESCROLL,0,
                                            SendMessage(GetDlgItem(hdlg,IDC_TXT_LOG),EM_GETLINECOUNT,0,0));
                            }
                        }                       
			        }
			        already_reading=FALSE;
		        }
            }		    
            else vRecvBuf->start=vRecvBuf->end; // simply empty the buffer if not in logging mode

	        break;

	    case WM_HELLO:
            {           
                vti_started_first = FALSE;
                otherWnd=(HWND)lParam;
                CheckDlgButton(hdlg,IDC_CHK_VTI_RUNNING,TRUE);                
                ATOM a = GlobalAddAtom(vLinkFileName);
                SendMessage(otherWnd,WM_SEND_BUFFER,0,(LPARAM)a);
                GlobalDeleteAtom(a);                

                // because of a VTI's bug, the link is broken if VTI is started
                // after VTI-Logger. Here's a workaround: if the link is enabled
                // right after VTI-Logger received WM_HELLO, the link would work
                SendMessage(otherWnd,WM_ENABLE_LINK,0,0);
                ActivateLink();
            }
            break;

        case WM_GOODBYE:
	        SendMessage(hwndMain,WM_DISABLE_LINK,0,0);
            CheckDlgButton(hdlg,IDC_CHK_VTI_RUNNING,FALSE);
            otherWnd=NULL;            
            break;

	    case WM_ENABLE_LINK:
            ActivateLink();
            break;

	    case WM_DISABLE_LINK:
            DeactivateLink();
            break;

	    case WM_SEND_BUFFER:
            {
                char name[33];                
                GlobalGetAtomName((ATOM)lParam,name,32);
                HANDLE h = OpenFileMapping(FILE_MAP_ALL_ACCESS,FALSE,name);
                vRecvBuf=(LinkBuffer*)MapViewOfFile(h,FILE_MAP_ALL_ACCESS,0,0,sizeof(LinkBuffer));                
            }
            break;

	    default:
		    return DefWindowProc(hwnd,msg,wParam,lParam);
	}
	return 0;
}


//-------------------------------------------
//
//
BOOL CALLBACK DlgProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uMsg) 
    {
        case WM_SYSCOMMAND:
            switch(wParam)
            {
                case SC_ABOUT:
                    DialogBox(hInst,MAKEINTRESOURCE(IDD_ABOUT),hwnd,AboutProc);
                    break;

                case SC_MINIMIZE:                    
                    SendMessage(hwndMain,WM_USER_MINIMIZE,wParam,lParam);                    
                    return TRUE;

                case SC_MAXIMIZE:
                    SendMessage(hwndMain,WM_USER_MAXIMIZE,wParam,lParam);
                    break;

                case SC_RESTORE:;
                    SendMessage(hwndMain,WM_USER_RESTORE,wParam,lParam);
                    break;
            }    
            break;

        case WM_SIZE:
            if ((lastWidth!=0) && (lastHeight!=0) && (wParam != SIZE_MINIMIZED))
            {                
                // set the new location for checkbox & button base on
                // the size's delta 
                SHORT deltaX = LOWORD(lParam)-lastWidth;
                SHORT deltaY = HIWORD(lParam)-lastHeight;
                lastWidth = LOWORD(lParam);
                lastHeight = HIWORD(lParam);

                SetDlgItemPos(GetDlgItem(hwnd,IDC_BTN_LOG),deltaX,0,0,0);
                SetDlgItemPos(GetDlgItem(hwnd,IDC_BTN_CLEAR),deltaX,0,0,0);
                SetDlgItemPos(GetDlgItem(hwnd,IDC_CHK_ONTOP),deltaX,0,0,0);
                SetDlgItemPos(GetDlgItem(hwnd,IDC_CHK_FILENAME),0,deltaY,0,0);
                SetDlgItemPos(GetDlgItem(hwnd,IDC_TXT_FILENAME),0,deltaY,deltaX,0);
                SetDlgItemPos(GetDlgItem(hwnd,IDC_CHK_APPEND),0,deltaY,0,0);
                SetDlgItemPos(GetDlgItem(hwnd,IDC_CHK_OPEN),0,deltaY,0,0);
                SetDlgItemPos(GetDlgItem(hwnd,IDC_BTN_BROWSE),deltaX,deltaY,0,0);
                SetDlgItemPos(GetDlgItem(hwnd,IDC_TXT_LOG),0,0,deltaX,deltaY);
                return TRUE;
            }
            break;

        case WM_GETMINMAXINFO:
            if ((minWidth>0) && (minHeight>0)) 
            {
                MINMAXINFO* pMinMax = (MINMAXINFO*) lParam;
                pMinMax->ptMinTrackSize.x = minWidth;
                pMinMax->ptMinTrackSize.y = minHeight;
                return TRUE;
            }
            break;

        case WM_COMMAND:
		    switch(wParam)
            {
                case IDC_BTN_CLEAR:
                    SetDlgItemText(hwnd,IDC_TXT_LOG,"");
                    break;

		        case IDC_BTN_BROWSE:
                    {
                        char currentDir[1024];
                        GetCurrentDirectory(1023,currentDir);
                        openDlgData.lpstrInitialDir = currentDir;
                        GetDlgItemText(hwnd,IDC_TXT_FILENAME,logfile,1023);
                        if (GetOpenFileName(&openDlgData)==TRUE)                    
                            SetDlgItemText(hwnd,IDC_TXT_FILENAME,logfile);                    
                    }
			        break;

                case IDC_BTN_LOG:
                    if ((IsDlgButtonChecked(hwnd,IDC_CHK_SCREEN)!=BST_CHECKED) &&
                        (IsDlgButtonChecked(hwnd,IDC_CHK_FILENAME)!=BST_CHECKED))
                    {
                        MessageBox(hwnd,"You must specify at least one ouput.","Error",MB_OK|MB_ICONWARNING);
                        break;
                    }

                    if (logging_enabled)
                    {
                        if (fp!=NULL)
                        {
                            fclose(fp);
                            fp = NULL;

                            // open the log file using notepad if selected
                            if (IsDlgButtonChecked(hdlg,IDC_CHK_OPEN)==BST_CHECKED)
                            {
                                strcpy (logfile+strlen(logfile)+1,"notepad.exe ");
                                strcat(logfile+strlen(logfile)+1,logfile);
                                WinExec(strcpy(logfile,logfile+strlen(logfile)+1),SW_SHOW);
                            }
                        }                        
                        EnableWindow(GetDlgItem(hwnd,IDC_CHK_SCREEN),TRUE);
                        EnableWindow(GetDlgItem(hwnd,IDC_CHK_FILENAME),TRUE);
                        EnableWindow(GetDlgItem(hwnd,IDC_BTN_CLEAR),TRUE);
                        SetDlgItemText(hwnd,IDC_BTN_LOG,"Start logging");
                        logging_enabled = FALSE;
                        if ((IsDlgButtonChecked(hwnd,IDC_CHK_VTI_RUNNING) == BST_CHECKED) &&
                            vti_started_first)
                        {
                            SendMessage(otherWnd,WM_DISABLE_LINK,0,0);
                            DeactivateLink();
                        }
                    }
                    else
                    {
                        // fp must be == NULL prior to this
                        if (IsDlgButtonChecked(hwnd,IDC_CHK_FILENAME) == BST_CHECKED)
                        {
				            GetDlgItemText(hwnd,IDC_TXT_FILENAME,logfile,1023);

				            if (logfile[0] != 0)
                            {                            
					            if (IsDlgButtonChecked(hwnd,IDC_CHK_APPEND) == BST_CHECKED) 
						            fp = fopen(logfile,"a+b");
					            else 
						            fp = fopen(logfile,"wb");
                            
                                if (fp == NULL) 
                                {
						            MessageBox(hwnd,"Cannot open logfile.","Error",MB_OK|MB_ICONWARNING);
                                    break;
                                }
                            }	
                            else
                            {
                                MessageBox(hwnd,"You must specify a filename.","Error",MB_OK|MB_ICONWARNING);
                                break;
                            }
                        }
                        EnableWindow(GetDlgItem(hwnd,IDC_CHK_SCREEN),FALSE);
                        EnableWindow(GetDlgItem(hwnd,IDC_CHK_FILENAME),FALSE);						
                        EnableWindow(GetDlgItem(hwnd,IDC_BTN_CLEAR),FALSE);
                        SetDlgItemText(hwnd,IDC_BTN_LOG,"Stop logging");
						logging_enabled = TRUE;
						if ((IsDlgButtonChecked(hwnd,IDC_CHK_VTI_RUNNING) == BST_CHECKED) &&
                           vti_started_first)
                        {   
                            SendMessage(otherWnd,WM_ENABLE_LINK,0,0);
                            ActivateLink();
                        }
                    }                    
                    SendMessage(hwnd,WM_COMMAND,IDC_CHK_SCREEN,0);
                    SendMessage(hwnd,WM_COMMAND,IDC_CHK_FILENAME,0);
				    break;

                case IDC_CHK_SCREEN:
                    {
                        BOOL status = (IsDlgButtonChecked(hwnd,IDC_CHK_SCREEN)==BST_CHECKED);
                        EnableWindow(GetDlgItem(hwnd,IDC_TXT_LOG),status);
                        EnableWindow(GetDlgItem(hwnd,IDC_BTN_CLEAR),status && IsWindowEnabled(GetDlgItem(hwnd,IDC_CHK_SCREEN)));
                    }
                    break;

                case IDC_CHK_FILENAME:
                    {
                        BOOL status = IsWindowEnabled(GetDlgItem(hwnd,IDC_CHK_FILENAME)) &&
                                      (IsDlgButtonChecked(hwnd,IDC_CHK_FILENAME)==BST_CHECKED);
                        EnableWindow(GetDlgItem(hwnd,IDC_CHK_APPEND),status);
                        EnableWindow(GetDlgItem(hwnd,IDC_CHK_OPEN),status);
                        EnableWindow(GetDlgItem(hwnd,IDC_TXT_FILENAME),status);
                        EnableWindow(GetDlgItem(hwnd,IDC_BTN_BROWSE),status);
                    }
                    break;

                case IDC_CHK_ONTOP:
                    if (IsDlgButtonChecked(hwnd,IDC_CHK_ONTOP) == BST_CHECKED)
                        SetWindowPos(hwnd,HWND_TOPMOST,0,0,0,0,SWP_NOSIZE | SWP_NOMOVE);
                    else
                        SetWindowPos(hwnd,HWND_NOTOPMOST,0,0,0,0,SWP_NOSIZE | SWP_NOMOVE);
			        break;
            }            
		    return TRUE;

        case WM_CLOSE:
		    SendMessage(hwndMain,uMsg,wParam,lParam);
		    return TRUE;

        // return the window background color the the log
        case WM_CTLCOLORSTATIC:
            if ((HWND)lParam == GetDlgItem(hwnd,IDC_TXT_LOG) &&
                (IsDlgButtonChecked(hwnd,IDC_CHK_SCREEN)==BST_CHECKED))

				return (BOOL)SendMessage(hwnd,WM_CTLCOLOREDIT,wParam,lParam);
		    break;        
    }
    return(FALSE);
}


//-------------------------------------------
//
//
BOOL CALLBACK AboutProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{    
    if (uMsg == WM_CLOSE)
    {
        EndDialog(hwnd,0);
        return TRUE;
    }
    else
        return FALSE;
}

//-------------------------------------------
//
//
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;
    int i;

	hInst = hInstance;    

    for (i=0;;i++) 
    {
        sprintf(vLinkFileName,"Virtual Link %d",i);
        hVLinkFileMap=CreateFileMapping((HANDLE)-1,NULL,
            PAGE_READWRITE,0,sizeof(LinkBuffer),vLinkFileName);
        if (GetLastError()!=ERROR_ALREADY_EXISTS)
            break;
    }

    vSendBuf=(LinkBuffer*)MapViewOfFile(hVLinkFileMap,
                                        FILE_MAP_ALL_ACCESS,
										0,
										0,
										sizeof(LinkBuffer));

    hMutex=CreateMutex(NULL,TRUE,"Virtual TI");

    if (GetLastError()==ERROR_ALREADY_EXISTS)
    {
		otherWnd=FindWindow("TEmuWnd",NULL);
        vti_started_first = TRUE;
    }
    else
        vti_started_first = FALSE;

	if (!InitApplication()) return 0;
	if ((hwndMain = CreateMainWindow()) == (HWND)0) return 0;
    
    hdlg = CreateDialog(hInst,
                        MAKEINTRESOURCE(IDD_MAIN),
                        hwndMain,
	  		            DlgProc);    
    CheckDlgButton(hdlg,IDC_CHK_VTI_RUNNING,(otherWnd != NULL));
    CheckDlgButton(hdlg,IDC_CHK_SCREEN,TRUE);

    // add About menuitem in System menus
    HMENU sysMenu = GetSystemMenu(hwndMain,FALSE);
    AppendMenu(sysMenu,MF_SEPARATOR,0,"");
    AppendMenu(sysMenu,MF_STRING,SC_ABOUT,"About...");

    sysMenu = GetSystemMenu(hdlg,FALSE);
    AppendMenu(sysMenu,MF_SEPARATOR,0,"-");
    AppendMenu(sysMenu,MF_STRING,SC_ABOUT,"&About...");

    // set the max size of the log
    SendMessage(GetDlgItem(hdlg,IDC_TXT_LOG),EM_SETLIMITTEXT,MAX_TEXT_BUFFER,0);

    // set the icon for the dialog window
    SendMessage(hdlg,WM_SETICON,ICON_BIG,(LPARAM)LoadIcon(hInst,(LPCTSTR)IDI_VTILOG));	

    // display
    ShowWindow(hwndMain,SW_SHOW);
	ShowWindow(hdlg,nCmdShow);    

    // save the dialog's width and height
    RECT rect;
    GetClientRect(hdlg,&rect);
    lastWidth = (WORD)rect.right;
    lastHeight = (WORD)rect.bottom;    

    GetWindowRect(hdlg,&rect);
    minWidth = (WORD)(rect.right-rect.left);
    minHeight = (WORD)(rect.bottom-rect.top);

    // increase the window's height using a factor of 1.3
    SetWindowPos(hdlg,0,0,0,minWidth,WORD(minHeight*1.3),SWP_NOMOVE|SWP_NOREPOSITION);

    // prepare Open dialog
    ZeroMemory(&openDlgData, sizeof(OPENFILENAME));
    openDlgData.lStructSize = sizeof(OPENFILENAME);
    openDlgData.hwndOwner = hdlg;
    openDlgData.lpstrFile = logfile;
    openDlgData.nMaxFile = sizeof(logfile);
    openDlgData.lpstrFilter = "Text files (*.txt;*.log)\0*.txt;*.log\0All files (*.*)\0*.*\0";
    openDlgData.nFilterIndex = 1;
    openDlgData.lpstrFileTitle = NULL;
    openDlgData.nMaxFileTitle = 0;
    openDlgData.lpstrInitialDir = NULL;
    openDlgData.lpstrDefExt = "txt";
    openDlgData.Flags = OFN_PATHMUSTEXIST | OFN_NOREADONLYRETURN | OFN_HIDEREADONLY;

    if (otherWnd)
    {
        SendMessage(otherWnd,WM_HELLO,0,(LPARAM)hwndMain);
        ATOM a=GlobalAddAtom(vLinkFileName);
        SendMessage(otherWnd,WM_SEND_BUFFER,0,(LPARAM)a);
        GlobalDeleteAtom(a);
    }

    while (GetMessage(&msg,NULL,0,0)) 
    {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
    }
    return msg.wParam;
}



